{
   Program_12_Main.Spin
   Implements the new extended protocol
   With all the new hardware
   With the allowance for Parameter setting
   
}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  RB_TX          = 1
  RB_RX          = 0
  RB_BAUD        = 115200
  D_TX           = 30
  D_RX           = 31
  D_BAUD         = 115200
  DimmerPin      = 23
  OthersFlagMask = %0000_0001  'flag mask for flagging the Others Cog
  MotorsFlagMask = %0000_0010  'flag mask for flagging the Motors Cog
  DimmerScale    = 16_777_216  ' 2 256   for DAC counter
                               ' to make dimming to be 0 to 255
Var
   byte SendBufferOffset
OBJ
  RB     : "FullDuplexSerial"
  D      : "SerialMirror"
  Others : "Program_11_Others"  
  Reader : "Program_11_Reader"   
  Motors : "Program_12_Motors"   

PUB Main|x
  Initialization
  repeat
    SendBufferOffset~   'reset data buffer pointer
    ReceiveCommand      'receive 2 bytes command,parameter
    OutputToPST_1       'output some info to the PST
    ExecuteTheCommand   'Execute the Command
    SendTheBuffer       'send the data buffer (either the
                        'normal or the secondary depending
                        'on the command just executed
    OutputToPST_2       'output some info to the PST

Dat
  Command         Byte 0,0         'command and parameter from RB
  PrimaryBuffer   Byte 0,0,0,0,0   'primary 5 bytes to send to RB
  SecondaryBuffer Byte 0,0,0,0,0   'secondary 5 bytes to send to RB
  Sems_Flags      byte 0,0,0       'semaphore1,flags,semaphore2 
  Settings        Long 80_000_000  'various settings buffer
                                    '[0] is Reader blinker duration
  TimeOut1        Long 240_000_000 '3 secs time out for receiving second byte
  TimeOut2        Long 320_000_000 '4 secs time out for being allowed to send data
  
Pri Initialization
  RB.Start(RB_RX,RB_TX,0,RB_BAUD)    'start FDS serial driver
  D.Start(D_RX,D_TX,0,D_BAUD)        'start SM serial driver (only need to do once)
  Sems_Flags[0] := LockNew           'create Semaphore for SendBuffer
  Sems_Flags[2] := LockNew           'create Semaphore for PST output
  Others.Start(@Command,@Settings,@Sems_Flags)        'start Others Cog
  Reader.Start(@PrimaryBuffer,@Settings,@Sems_Flags)  'Start Reader Cog
  Motors.Start(@Command,@Settings,@Sems_Flags)        'Start Motors Cog

  'create a dimmer LED on P23
  ctrA[30..26] := %00110            'Set ctra to DUTY mode
  ctrA[5..0]   := DimmerPin         'Set ctra's APIN
  frqA := 127 *DimmerScale          'Set frqa register to mid level to start with
  dirA[DimmerPin]~~                 'make output pin

Pri ReceiveCommand|t,x
  repeat
     Sems_Flags[1]~          'reset flags
     repeat                  'wait forever to receive command byte
         Process0            'call process that need to be done always
         x := RB.RXcheck       
     until x <> -1           
     Command[0] := x
     t := cnt                'init timer
     repeat                  'wait with timeout to receive parameter
         Process0            'call process that needs to be done always
         x := RB.RXcheck     
         if cnt-t > TimeOut1
             quit  
     until x <> -1           
     if x == -1              'if no byte received loop back to the top
        Next                 'gracefully recover from receive error
     Command[1] := x
     quit                    'exit to return to Main

Pri SendTheBuffer|x
  if Not Sems_Flags[1]       'if flags clear then send bytes
    'if SendBufferOffset == 0 'if using primary buffer wait for semaphore
    '                         'to ensure data congruity
    '   repeat until Not LockSet(Sems_Flags[0]) 
    '       'wait for semaphore forever, may need to do timeout but 
    '       'uncomment these lines to assure data conguity 
    '       'also the LockClr() below
    repeat x from 0 to 4      'transmit the 5 bytes of data     
      RB.TX(PrimaryBuffer[x+SendBufferOffset])
    'if SendBufferOffset == 0   'if data congruity with semaphore is used
    '   LockClr(Sems_Flags[0])  'then also uncomment out these two line also
 
Pri Process0
  'can do anything here.
  'but there is nothing to do in this case
  'but could anything that requires polling or updating etc.

Pri ExecuteTheCommand
  case Command[0]
    255: 'reset the propeller
      Reboot
      
    200:'set the Dimmer level on P23
      frqA := Command[1] *DimmerScale
       
    201:'set blinker duration for Reader cog(P22)
        '256 levels between 0 to 2 secs (0=off)               
      Settings[0] := Command[1]*(2_000/256)*(ClkFreq/1_000)

    202: 'set the TimeOut1 for waiting for parameter byte (second command byte)
         'in seconds
         TimeOut1 := (clkfreq/1000)*Command[1]*500
    203: 'set the TimeOut2 for waiting for a command to finish it sactions
         'so as to send the send buffer it should reflect and be in relations
         'to the PC system's time out
         'in seconds
         TimeOut2 := (clkfreq/1000)*Command[1]*500
      
    1,2,192:'commands in the Others cog that may fill the
          'the last two bytes in the primary send buffer
      Sems_Flags[1] |= OthersFlagMask    'set flag
      WaitForFlagResetWithTimeOut(OthersFlagMask)
      
    66:'commands in the Others cog that fill the secondary send buffer
      SendBufferOffset := 5            'command uses the secondary buffer
      Sems_Flags[1] |= OthersFlagMask  'set flag to signal the cog
      WaitForFlagResetWithTimeOut(OthersFlagMask)

    0,6,7,12,13,240,241,242,243,244,245:'commands in the Motors cog
      Sems_Flags[1] |= MotorsFlagMask    'set flag
      WaitForFlagResetWithTimeOut(MotorsFlagMask)
         
Pri WaitForFlagResetWithTimeOut(Flag)|t
  t:=cnt
  repeat until cnt-t > TimeOut2            'wait for flag to clear with time out
     if Not(Sems_Flags[1] & Flag)
        quit 

Pri OutputToPST_1
  if Not LockSet(Sems_Flags[2])  'output some info to the PST if semaphore acquired
    D.Str(string(13,"Main Received:"))
    D.Dec(Command[0])
    D.Str(string(","))
    D.Dec(Command[1])
    LockClr(Sems_Flags[2])

Pri OutputToPST_2|x
  if Not LockSet(Sems_Flags[2]) 'output some info to the PST if semaphore acquired
    D.Str(string(13,"Main Sending:"))
    repeat x from 0 to 4        'show the bytes sent            
       D.Dec(PrimaryBuffer[x+SendBufferOffset])
       D.Str(string(","))
    LockClr(Sems_Flags[2])