{
   Program_12_Motors.Spin
   if a flag is set Executes various commands
   relating to motors
   Implements the new Stepp size and Turn size
   as well as parameter setting. 
}
CON
  FlagMask  = %0000_0010   'mask for flag for this cog
  Offset_1  = 5   'offset for loaction of 2 byte data buffer    
  Offset_2  = 7   'offset for location of 5 byte data buffer
  MidVal    = 1500
  MinVal    = 1000
  MaxVal    = 2000
  Servo1    = 16
  Servo2    = 17

Var
  long Stacks[50], t, onTimer, MotorsOn, MotorsTimeOut
  Long Command,Settings,Sems_Flags 

Dat
  L_Speed     long 255  'speed factor for moving forward/backward
  T_Speed     long 255  'speed factor for turning    
  StepTime    long 30
  TurnTime    long 20
  L_TimeOut   long 100  'motor keep on time after forward/backward
  T_TimeOut   long 0    'motor keep on time after turning

Obj
  D      : "SerialMirror"
  Servo  : "Servo32V7"
  
PUB Start(CommandAddr,SettingsAddr,Sems_FlagsAddr)
  Command    :=   CommandAddr    'save all passed addresses as     
  Settings   :=   SettingsAddr   'local variables to be used
  Sems_Flags :=   Sems_FlagsAddr 'by all processes
  cogNew(Motors,@Stacks[0])     'start the cog 

Pri Motors
  Initialization
  repeat
     Process0
     if Not(byte[Sems_Flags][1] & FlagMask) 'if flag not set then no action
        Next                                'required, just loop back                            
     case byte[Command][0]                  'execute the command
         0:  StopMotors(0)
         6:  MoveMotors(1)  'forward
         7:  MoveMotors(-1) 'back
        12:  TurnMotors(1)  'right
        13:  TurnMotors(-1) 'left
       240:  L_Speed   := byte[Command][1]    'set speed 
       241:  L_TimeOut := byte[Command][1]*10 'set timeout in steps of 10ms  
       242:  StepTime  := byte[Command][1]*10 'set step time in steps of 10ms  
       243:  TurnTime  := byte[Command][1]*10 'set turn time in steps of 10ms  
       244:  T_Speed   := byte[Command][1]    'set speed 
       245:  T_TimeOut := byte[Command][1]*10 'set timeout in steps of 10ms  

     OutputToPST                        'output some info to the PST
     byte[Sems_Flags][1] &= !FlagMask   'clear the flag to signal to Main to proceed

Pri Initialization
  'initialize variables I/O direction etc.
  'none needed here
  t := cnt                        'initialize timer
  Servo.Start
  MotorsOn~

Pri Process0
  'can do anything here that requires polling or updating etc.
  'We will use this to turn motors off after a certain timeout
  if MotorsOn And cnt-onTimer > MotorsTimeOut*(clkfreq/1000)
     StopMotors(0)
  
Pri StopMotors(wait)
  Servo.Set(Servo1,MidVal)
  Servo.Set(Servo2,MidVal)
  MotorsOn~
  if wait
    waitcnt(cnt+(clkfreq/1_000_000)*50)  'wait 50 microsecs

Pri MoveMotors(Dir)|Val1,Val2
  StopMotors(1)
  Val1 := MidVal+Dir*1000*L_Speed/255
  Val2 := MidVal-Dir*1000*L_Speed/255
  if byte[Command][1]
    MotorsOn~~     
    Servo.Set(Servo1,Val1)
    Servo.Set(Servo2,Val2)
    WaitForDistance

Pri WaitForDistance
  'this will use whatever system needed to evaluate the
  'the distance moved. For now we will just use a timed step
  waitcnt(clkfreq/1000*StepTime*byte[Command][1]+cnt)
  MotorsTimeOut := L_TimeOut
  OnTimer := cnt

Pri TurnMotors(Dir)|Val1,Val2
  StopMotors(1)
  Val1 := MidVal+Dir*1000*T_Speed/255
  Val2 := MidVal+Dir*1000*T_Speed/255
  if byte[Command][1]
    MotorsOn~~
    Servo.Set(Servo1,Val1)
    Servo.Set(Servo2,Val2)
    WaitForAngle

Pri WaitForAngle
  'this will use whatever system needed to evaluate the
  'the angle turned. For now we will just use a timed turn
  waitcnt(clkfreq/1000*TurnTime*byte[Command][1]+cnt)
  MotorsTimeOut := T_TimeOut
  OnTimer := cnt

Pri OutputToPST
  if Not LockSet(byte[Sems_Flags][2]) 'output some info to the PST
     D.Str(string(13,"Motors:"))      'if semaphore acquired
     D.Dec(byte[Command][0])   
     D.Str(string(","))
     D.Dec(byte[Command][1])   
     LockClr(byte[Sems_Flags][2])
           