{
   Firmware_Main_E.Spin
   Implements the new extended protocol
   With all the new hardware+Compass
   With the allowance for Parameter setting
   Also setting motors individually
   Also allowance for a Turret
   Also EEPROM writing/reading
   and H48C accelerometer
   
}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  RB_TX          = 1
  RB_RX          = 0
  RB_BAUD        = 115200
  D_TX           = 30
  D_RX           = 31
  D_BAUD         = 115200
  DimmerPin      = 23
  OthersFlagMask = %0000_0001  'flag mask for flagging the Others Cog
  MotorsFlagMask = %0000_0010  'flag mask for flagging the Motors Cog
  DimmerScale    = 16_777_216  ' 2 256   for DAC counter
                               ' to make dimming to be 0 to 255
  eepromAddress     = $7000    'adress in EEProm to write data buffer
  BufferSize        = 64       'has to be multiples of 32
  MainLongsToSave   = 5        'number of longs to store in EEPROM  from Main                       
  MotorsLongsToSave = 6        'number of longs to store in EEPROM  from Motrs                       

  'factory settings
  F_P22_Blinker  = 80_000_000  'is Reeader blinker duration
  F_P21_Blinker  = 10          'is Others blinker freq
  F_P23_Dimmer   = 127         'is Main dimmer level
  F_TimeOut1     = 240_000_000 '3 secs time out for receiving second byte
  F_TimeOut2     = 320_000_000 '4 secs time out for being allowed to send data


Var
   byte SendBufferOffset
   Long Buffer[BufferSize/4]  'read/write buffer for the EEPROM
OBJ
  RB     : "FullDuplexSerial"
  D      : "SerialMirror"
  I2C    : "Basic_I2C_Driver"
  Others : "Firmware_Others_E"  
  Reader : "Firmware_Reader"   
  Motors : "Firmware_Motors_D"   

PUB Main|x
  Initialization
  repeat
    SendBufferOffset~   'reset data buffer pointer
    ReceiveCommand      'receive 2 bytes command,parameter
    OutputToPST_1       'output some info to the PST
    ExecuteTheCommand   'Execute the Command
    SendTheBuffer       'send the data buffer (either the
                        'normal or the secondary depending
                        'on the command just executed
    OutputToPST_2       'output some info to the PST

Dat
  Command         Byte 0,0         'command and parameter from RB
  PrimaryBuffer   Byte 0,0,0,0,0   'primary 5 bytes to send to RB
  SecondaryBuffer Byte 0,0,0,0,0   'secondary 5 bytes to send to RB
  Sems_Flags      byte 0,0,0       'semaphore1,flags,semaphore2 
  Settings        Long F_P22_Blinker 'various settings buffer
                                     '[0] is Reader blinker duration
                  Long F_P21_Blinker '[1] is Others blinker freq
                  Long F_P23_Dimmer  '[2] is Main dimmer level
  TimeOut1        Long F_TimeOut1    '3 secs time out for receiving second byte
  TimeOut2        Long F_TimeOut2    '4 secs time out for being allowed to send data
  
Pri Initialization
  RB.Start(RB_RX,RB_TX,0,RB_BAUD)    'start FDS serial driver
  D.Start(D_RX,D_TX,0,D_BAUD)        'start SM serial driver (only need to do once)
  Sems_Flags[0] := LockNew           'create Semaphore for SendBuffer
  Sems_Flags[2] := LockNew           'create Semaphore for PST output

  RestoreSettings    'read settings from EEPROM if any available

  Others.Start(@Command,@Settings,@Sems_Flags)        'start Others Cog
  Reader.Start(@PrimaryBuffer,@Settings,@Sems_Flags)  'Start Reader Cog
  Motors.Start(@Command,@Settings,@Sems_Flags)        'Start Motors Cog
  'create a dimmer LED on P23
  ctrA[30..26] := %00110            'Set ctra to DUTY mode
  ctrA[5..0]   := DimmerPin         'Set ctra's APIN
  frqA := Settings[2] *DimmerScale          'Set frqa register to mid level to start with
  dirA[DimmerPin]~~                 'make output pin

Pri ReceiveCommand|t,x
  repeat
     Sems_Flags[1]~          'reset flags
     repeat                  'wait forever to receive command byte
         Process0            'call process that need to be done always
         x := RB.RXcheck       
     until x <> -1           
     Command[0] := x
     t := cnt                'init timer
     repeat                  'wait with timeout to receive parameter
         Process0            'call process that needs to be done always
         x := RB.RXcheck     
         if cnt-t > TimeOut1
             quit  
     until x <> -1           
     if x == -1              'if no byte received loop back to the top
        Next                 'gracefully recover from receive error
     Command[1] := x
     quit                    'exit to return to Main

Pri SendTheBuffer|x
  if Not Sems_Flags[1]       'if flags clear then send bytes
    'if SendBufferOffset == 0 'if using primary buffer wait for semaphore
    '                         'to ensure data congruity
    '   repeat until Not LockSet(Sems_Flags[0]) 
    '       'wait for semaphore forever, may need to do timeout but 
    '       'uncomment these lines to assure data congruity 
    '       'also the LockClr() below
    repeat x from 0 to 4      'transmit the 5 bytes of data     
      RB.TX(PrimaryBuffer[x+SendBufferOffset])
    'if SendBufferOffset == 0   'if data congruity with semaphore is used
    '   LockClr(Sems_Flags[0])  'then also uncomment out these two line also
 
Pri Process0
  'can do anything here.
  'but there is nothing to do in this case
  'but could be anything that requires polling or updating etc.

Pri ExecuteTheCommand|x ,a
  case Command[0]
      4: SendSystemParameter  'send requested parameter to RB
      5: StorePrintSystemParameters  'store parameters to EEPROM or print them to the PST       

    255: Reboot 'reset the propeller

    200:'set the Dimmer level on P23
        frqA := Command[1] *DimmerScale
        Settings[2] := Command[1]

    201:'set blinker duration for Reader cog(P22)
        '256 levels between 0 to 2 secs (0=off)               
      Settings[0] := Command[1]*(2_000/256)*(ClkFreq/1_000)

    202: 'set the TimeOut1 for waiting for parameter byte (second command byte)
         'in seconds
         TimeOut1 := (clkfreq/1000)*Command[1]*500
    203: 'set the TimeOut2 for waiting for a command to finish its actions
         'so as to send the send buffer it should reflect and be in relations
         'to the PC system's time out
         'in seconds
         TimeOut2 := (clkfreq/1000)*Command[1]*500
      
    1,2,24:
      'commands in the Others cog that may fill the
      'the last two bytes in the primary send buffer
      Sems_Flags[1] |= OthersFlagMask    'set flag
      WaitForFlagResetWithTimeOut(OthersFlagMask)
      
    66:'commands in the Others cog that fill the secondary send buffer
      SendBufferOffset := 5            'command uses the secondary buffer
      Sems_Flags[1] |= OthersFlagMask  'set flag to signal the cog
      WaitForFlagResetWithTimeOut(OthersFlagMask)

    0,6,7,12,13,240,241,242,243,244,245,8,9,10,11:'commands in the Motors cog
      Sems_Flags[1] |= MotorsFlagMask    'set flag
      WaitForFlagResetWithTimeOut(MotorsFlagMask)

    192,193: 'turret and ping commands in Motors and Others cogs
      Sems_Flags[1] |= MotorsFlagMask    'set flag for Motors
      WaitForFlagResetWithTimeOut(MotorsFlagMask)
      Command[0] := 192  'force the command code to 192, this way we do not need to
                         'to change the Others object
      Sems_Flags[1] |= OthersFlagMask    'set flag for Others
      WaitForFlagResetWithTimeOut(OthersFlagMask)

    70:'H48C if parameter is 1=>read vref and any other value means read the axes 
      if Command[1] <> 1
         SendBufferOffset := 5            'command uses the secondary buffer
      Sems_Flags[1] |= OthersFlagMask  'set flag to signal the cog
      WaitForFlagResetWithTimeOut(OthersFlagMask)

         
Pri WaitForFlagResetWithTimeOut(Flag)|t
  t:=cnt
  repeat until cnt-t > TimeOut2            'wait for flag to clear with time out
     Process0
     if Not(Sems_Flags[1] & Flag)
        quit 

Pri RestoreSettings|x,crc,a
  if i2c.ReadPage(i2c#BootPin, i2c#EEPROM, eepromAddress, @Buffer, BufferSize)
     return 'could not write
  a := MainLongsToSave+MotorsLongsToSave    
  if Buffer[a] <> a 'bad count
     return
  crc~   
  repeat x from 0 to a-1  'create crc
     crc += Buffer[x]
  if crc <> Buffer[a+1]   'bad crc
     return
  repeat x from 0 to MainLongsToSave-1    'copy buffer to Main data
     Settings[x] := Buffer[x]          
  a := Motors.GetDataAddress
  repeat x from 0 to MotorsLongsToSave-1  'copy buffer to Motors Data
     Long[a][x] := Buffer[x+MainLongsToSave] 
  
Pri StoreSettings|x,a,crc
  crc~
  PrimaryBuffer[3] := PrimaryBuffer[4] :=0  'assume failure
  repeat x from 0 to MainLongsToSave-1    'copy Main data to buffer
     Buffer[x] := Settings[x]
     crc += Settings[x]                   'calc crc
  a := Motors.GetDataAddress
  repeat x from 0 to MotorsLongsToSave-1  'copy Motors Data to buffer
     Buffer[x+MainLongsToSave] := Long[a][x]
     crc += Long[a][x]                    'calc crc
  x := MainLongsToSave+MotorsLongsToSave   
  Buffer[x] := x  'store count
  Buffer[x+1] := crc  'store crc            
  if i2c.WritePage(i2c#BootPin, i2c#EEPROM, eepromAddress, @Buffer, BufferSize)
    return ' an error occured during the write
  x := cnt ' prepare to check for a timeout
  repeat while i2c.WriteWait(i2c#BootPin, i2c#EEPROM, eepromAddress)
    if cnt - x > clkfreq / 10
      return ' waited more than a 1/10 second for the write to finish
  PrimaryBuffer[4] :=1  'indicate success      

Pri OutputToPST_1
  if Not LockSet(Sems_Flags[2])  'output some info to the PST if semaphore acquired
    D.Str(string(13,"Main Received:"))
    D.Dec(Command[0])
    D.Str(string(","))
    D.Dec(Command[1])
    LockClr(Sems_Flags[2])

Pri OutputToPST_2|x
  if Not LockSet(Sems_Flags[2]) 'output some info to the PST if semaphore acquired
    D.Str(string(13,"Main Sending:"))
    repeat x from 0 to 4        'show the bytes sent            
       D.Dec(PrimaryBuffer[x+SendBufferOffset])
       D.Str(string(","))
    LockClr(Sems_Flags[2])

Pri SendSystemParameter|x,a,fifth
   'send RAM parameter to RB if not valid then all bytes are $FF
   'if valid then Little Endian for the first 4 bytes and $00 for fifth
    fifth := x := $FFFFFFFF  
    if Command[1] < MainLongsToSave  'if parameter request is in Main
        x := Settings[Command[1]]
        fifth := 0
    elseif Command[1] < MotorsLongsToSave+MainLongsToSave    'if in Motors  
        a := Motors.GetDataAddress
        x:= Long[a][Command[1]-MainLongsToSave]
        fifth := 0
    SendBufferOffset := 5  'signal that it is secondary send buffer
    SecondaryBuffer[4] := fifth & $FF    'set the last byte
    repeat a from 0 to 3                 'set the 4 bytes Litle Endian
       SecondaryBuffer[a] := x.byte[a]

Pri Restore_Factory
  Settings[0] := F_P22_Blinker
  Settings[1] := F_P21_Blinker
  Settings[2] := F_P23_Dimmer
  Settings[3] := F_TimeOut1
  Settings[4] := F_TimeOut2
  
Pri StorePrintSystemParameters|x,a
  case Command[1]
    0:StoreSettings  'write the setting to the eeprom
    1:Restore_Factory
      Motors.Restore_Factory 
    2:'output EEPOROM parameter to PST
      D.Tx(0)
      D.Str(string("Reading the EEPROM",13))
      ifnot i2c.ReadPage(i2c#BootPin,i2c#EEPROM,eepromAddress,@Buffer,BufferSize)
        repeat x from 0 to MotorsLongsToSave+MainLongsToSave-1
           D.Dec(Buffer[x])
           D.Tx(32)
        D.Tx(13)    
      else        
        D.Str(string("Operation Failed",13))
    other: 'output RAM parameters to PST
      D.Tx(0)
      D.Str(string("RAM parameters",13))
      a := Motors.GetDataAddress
      repeat x from 0 to MainLongsToSave-1
         D.Dec(Settings[x])
         D.Tx(32)
      repeat x from 0 to MotorsLongsToSave-1
         D.Dec(Long[a][x])
         D.Tx(32)
      D.Tx(13)    