{
   Firmware_Motors_D.Spin
   if a flag is set Executes various commands
   relating to motors
   Implements the new Step size and Turn size
   as well as parameter setting.

   Adds setting motors individually
   Adds Allowance for a turret
   Also for EEPROM read/write 
}
CON
  FlagMask  = %0000_0010   'mask for flag for this cog
  Offset_1  = 5   'offset for location of 2 byte data buffer    
  Offset_2  = 7   'offset for location of 5 byte data buffer
  MidVal    = 1500
  MinVal    = 1000
  MaxVal    = 2000
  Servo1    = 16
  Servo2    = 17
  Servo3    = 15  'standard servo for turret
  TS_MaxVal = 800
  TS_MinVal = 800
  'Factory Settings
  F_L_Speed     = 255  'speed factor for moving forward/backward
  F_T_Speed     = 255  'speed factor for turning    
  F_StepTime    = 30
  F_TurnTime    = 20
  F_L_TimeOut   = 100  'motor keep on time after forward/backward
  F_T_TimeOut   = 30   'motor keep on time after turning
  

Var
  long CogN,Stacks[50], t, onTimer, MotorsOn, MotorsTimeOut
  Long Command,Settings,Sems_Flags
  Long LMotorOn,LMotorTimer,LMotorDuration,RMotorOn,RMotorTimer,RMotorDuration
  Long TurretPosition 

Dat
  L_Speed     long  F_L_Speed   'speed factor for moving forward/backward
  T_Speed     long  F_T_Speed   'speed factor for turning    
  StepTime    long  F_StepTime  
  TurnTime    long  F_TurnTime  
  L_TimeOut   long  F_L_TimeOut 'motor keep on time after forward/backward
  T_TimeOut   long  F_T_TimeOut 'motor keep on time after turning

Obj
  D      : "SerialMirror"
  Servo  : "Servo32V7"
  
PUB Start(CommandAddr,SettingsAddr,Sems_FlagsAddr)
  Command    :=   CommandAddr    'save all passed addresses as     
  Settings   :=   SettingsAddr   'local variables to be used
  Sems_Flags :=   Sems_FlagsAddr 'by all processes
  CogN := Result :=cogNew(Motors,@Stacks[0])+1     'start the cog 

Pub Stop
  If CogN
     CogStop(CogN~ -1)

Pub GetDataAddress  'return the address of the parameters buffer
   return @L_Speed

Pub Restore_Factory
  L_Speed    := F_L_Speed   'speed factor for moving forward/backward
  T_Speed    := F_T_Speed   'speed factor for turning    
  StepTime   := F_StepTime  
  TurnTime   := F_TurnTime  
  L_TimeOut  := F_L_TimeOut 'motor keep on time after forward/backward
  T_TimeOut  := F_T_TimeOut 'motor keep on time after turning

Pri Motors
  Initialization
  repeat
     Process0
     if Not(byte[Sems_Flags][1] & FlagMask) 'if flag not set then no action
        Next                                'required, just loop back                            
     case byte[Command][0]                  'execute the command
        00:  StopMotors(0)
        06:  MoveMotors(1)  'forward
        07:  MoveMotors(-1) 'back
        12:  TurnMotors(1)  'right
        13:  TurnMotors(-1) 'left
        08:  RightMotor(1)  'move Right motor separately forward
        09:  RightMotor(-1) 'move Right motor separately backward
        10:  LeftMotor(1)   'move Left  motor separately forward
        11:  LeftMotor(-1)  'move Left  motor separately backward
       192:  TurretSet(1)   'move the turret right
       193:  TurretSet(-1)  'move the turret left
       240:  L_Speed   := byte[Command][1]    'set speed 
       241:  L_TimeOut := byte[Command][1]*10 'set timeout in steps of 10ms  
       242:  StepTime  := byte[Command][1]*10 'set step time in steps of 10ms  
       243:  TurnTime  := byte[Command][1]*10 'set turn time in steps of 10ms  
       244:  T_Speed   := byte[Command][1]    'set speed 
       245:  T_TimeOut := byte[Command][1]*10 'set timeout in steps of 10ms  

     OutputToPST                        'output some info to the PST
     byte[Sems_Flags][1] &= !FlagMask   'clear the flag to signal to Main to proceed

Pri Initialization
  'initialize variables I/O direction etc.
  'none needed here
  t := cnt                   'initialize timer
  Servo.Start
  Servo.Set(Servo3,1500)     'make sure the turret is at center
  TurretPosition := 1500
  MotorsOn~

Pri Process0
  'can do anything here that requires polling or updating etc.
  'We will use this to turn motors off after a certain timeout
  if MotorsOn And cnt-onTimer > MotorsTimeOut*(clkfreq/1000)
     StopMotors(0)  'stop motors after timeout
  if LMotorOn And cnt-LMotorTimer > LMotorDuration  
     StopIndividualMotor(2) 'Left motor stop after steps run out
  if RMotorOn And cnt-RMotorTimer > RMotorDuration
     StopIndividualMotor(1) 'right motor stop after steps run out

Pri TurretSet(Dir)|x,xx
  if Dir < 0
     x := MidVal+TS_MaxVal*byte[Command][1]/90
  else   
     x := MidVal-TS_MinVal*byte[Command][1]/90
  Servo.Set(Servo3,x)
  xx := ||(TurretPosition-x) 'abs(difference)
  TurretPosition :=x
  if xx > 20 'only delay if more than 20ms  
     waitcnt(clkfreq/1600*xx+cnt) 
  'assuming it takes 1 sec to travel from 700 to 2300 then we delay a multile 1/1600 secs
  'depending how far it has to travel. 
  

Pri LeftMotor(Dir)|x
  StopIndividualMotor(2)   
  x := byte[Command][1]
  if x
     Servo.Set(Servo2,MidVal+Dir*1000*L_Speed/255)     
     if x <> 255 'on for distance
       LMotorOn~~
       LMotorTimer := cnt
       LMotorDuration :=StepTime*x*(clkfreq/1000)  'set time out

Pri RightMotor(Dir)|x
  StopIndividualMotor(1)   
  x := byte[Command][1]
  if x
     Servo.Set(Servo1,MidVal+Dir*1000*L_Speed/255)     
     if x <> 255 'on for distance
       RMotorOn~~
       RMotorTimer := cnt
       RMotorDuration :=StepTime*x*(clkfreq/1000)  'set time out

Pri StopIndividualMotor(which)
  if which==2  'right
     Servo.Set(Servo2,MidVal)
     LMotorOn~
  else  'left   
     Servo.Set(Servo1,MidVal)
     RMotorOn~
    
Pri StopMotors(wait)
  LMotorOn := RMotorOn := 0   'make sure separate movement is canceled
  Servo.Set(Servo1,MidVal)
  Servo.Set(Servo2,MidVal)
  MotorsOn~
  if wait
    waitcnt(cnt+(clkfreq/1_000_000)*50)  'wait 50 microsecs

Pri MoveMotors(Dir)|Val1,Val2
  StopMotors(1)
  Val1 := MidVal+Dir*1000*L_Speed/255
  Val2 := MidVal-Dir*1000*L_Speed/255
  if byte[Command][1]
    MotorsOn~~     
    Servo.Set(Servo1,Val1)
    Servo.Set(Servo2,Val2)
    WaitForDistance

Pri WaitForDistance
  'this will use whatever system needed to evaluate the
  'the distance moved. For now we will just use a timed step
  waitcnt(clkfreq/1000*StepTime*byte[Command][1]+cnt)
  MotorsTimeOut := L_TimeOut
  OnTimer := cnt

Pri TurnMotors(Dir)|Val1,Val2
  StopMotors(1)
  Val1 := MidVal+Dir*1000*T_Speed/255
  Val2 := MidVal+Dir*1000*T_Speed/255
  if byte[Command][1]
    MotorsOn~~
    Servo.Set(Servo1,Val1)
    Servo.Set(Servo2,Val2)
    WaitForAngle

Pri WaitForAngle
  'this will use whatever system needed to evaluate the
  'the angle turned. For now we will just use a timed turn
  waitcnt(clkfreq/1000*TurnTime*byte[Command][1]+cnt)
  MotorsTimeOut := T_TimeOut
  OnTimer := cnt

Pri OutputToPST
  if Not LockSet(byte[Sems_Flags][2]) 'output some info to the PST
     D.Str(string(13,"Motors:"))      'if semaphore acquired
     D.Dec(byte[Command][0])   
     D.Str(string(","))
     D.Dec(byte[Command][1])   
     LockClr(byte[Sems_Flags][2])

           