{
   Firmware_Others_D.Spin
   if a flag is set Executes various commands
   Pots, Ping))), Compass, Set LEDs and
   Allowance for EEPROM  
   
}
CON
  Blinker       = 21
  RCPin1        = 2
  RCPin2        = 3
  PingPin       = 4
  FlagMask      = %0000_0001   'mask for flag for this cog
  MotorsFlagMask= %0000_0010   'Mask For Motors Flag
  BlinkerScale  = 54           ' 2 80_000_000 for NCO counter
  Offset_1      = 5   'offset for location of 2 byte data buffer    
  Offset_2      = 7   'offset for location of 5 byte data buffer
  Compass_SDA   = 26
  Compass_SCL   = 27
  Compass_On    = 1   'if there is no compass make this 0
Var
  long CogN,Stacks[50], t
  byte Compass_Calibration
  Long Compass_Timer
  Long Command,Settings,Sems_Flags,RCTime[2] 

Obj
  D      : "SerialMirror"
  RC     : "Pots_RCTime"
  Compass: "HMC6352"                                
  
PUB Start(CommandAddr,SettingsAddr,Sems_FlagsAddr)
  Compass.Init(Compass_SDA, Compass_SCL)
  Command    :=   CommandAddr    'save all passed addresses as     
  Settings   :=   SettingsAddr   'local variables to be used
  Sems_Flags :=   Sems_FlagsAddr 'by all processes
  Compass_Calibration~
  CogN := Result :=cogNew(Others,@Stacks[0])+1      'start the cog 

Pub Stop
  If CogN
     CogStop(CogN~ -1)

Pri Others
  Initialization
  repeat
     Process0
     if Not(byte[Sems_Flags][1] & FlagMask) 'if flag not set then no action
        Next                                'required, just loop back                            
     case byte[Command][0]              'execute the command
          1: Set_LEDS
          2: Set_BlinkRate
         24: 'compass commands
             case byte[Command][1]
                1: Is_Compass_Enabled 'return if compass is available or not
                2: Calibrate_Compass(0)
                3: Calibrate_Compass(1)
                other: Read_Compass
         66: Read_Pots
        192: Read_Ping
     OutputToPST                        'output some info to the PST
     byte[Sems_Flags][1] &= !FlagMask   'clear the flag to signal to Main to proceed

Pri Initialization
  'initialize variables I/O direction etc.
  ctrA[30..26] := %00100          'Set ctrA for "NCO single-ended"
  ctrA[5..0]   := Blinker         'Set ctrA's APIN 
  frqA         := Long[Settings][1]*BlinkerScale 'set rate
  dirA[20..18]~~
  dirA[Blinker]~~
  t := cnt                        'initialize timer

Pri Process0
  'can do anything here.
  'but there is nothing to do in this case
  'but could anything that requires polling or updating etc.
  if Compass_Calibration                'if calibration is in progress
     if cnt-Compass_Timer > ClkFreq*20  'and time is out
        Compass.EndCalibration          'send end command
        Compass_Calibration~            'clear indicator
  
Pri Set_LEDs
  'set the LEDs on P20..P18 per the parameter byte
  outA[18..20] := Byte[Command][1]
  
Pri Set_BlinkRate
  'set the blink rate  for P21
  frqA := byte[Command][1] * BlinkerScale 'set rate
  Long[Settings][1] := byte[Command][1]
  if frqA == 0
     phsA~  'make sure the pin is low if it to be off             

Pri Read_Ping |x
  'read the Ping time and return its 2 byte value in the
  '4th and 5th bytes of the primary buffer. MSByte first.
  x := Ping(PingPin)
  byte[Command][Offset_1]   := (x >> 8) & $FF
  byte[Command][Offset_1+1] := x & $FF            

Pri Read_Pots
  'Read the two Pots and returns the readings in the
  'secondary send buffer. Use MSByte first for both
  'the 16 bit values
  RC.RCTime(RCPin1,1,@RCTime[0])   'use Pots_RCTime in direct mode ...no cog                                              
  RC.RCTime(RCPin2,1,@RCTime[1])   'to get one-time reading...                                              
  byte[Command][Offset_2]   := (RCTime[0]>>8)& $FF
  byte[Command][Offset_2+1] := RCTime[0] & $FF            
  byte[Command][Offset_2+2] := (RCTime[1]>>8)& $FF
  byte[Command][Offset_2+3] := RCTime[1] & $FF            
  byte[Command][Offset_2+4] := 0

Pri OutputToPST
  if Not LockSet(byte[Sems_Flags][2]) 'output some info to the PST
     D.Str(string(13,"Others:"))      'if semaphore acquired
     D.Dec(byte[Command][0])   
     D.Str(string(","))
     D.Dec(byte[Command][1])   
     LockClr(byte[Sems_Flags][2])

Pri Is_Compass_Enabled
  byte[Command][Offset_1]   := 0
  byte[Command][Offset_1+1] := Compass_On            

Pri Read_Compass |x
  'read the Compass and return its 2 byte value in the
  '4th and 5th bytes of the primary buffer. MSByte first.
  x~
  if Compass_on and Not Compass_Calibration
     x := Compass.Heading / 10
  byte[Command][Offset_1]   := (x >> 8) & $FF
  byte[Command][Offset_1+1] := x & $FF            

Pri Calibrate_Compass(Mode)
  'Starts the compass calibration
  'If the Mode is <> 0 then it will also
  'rotate the motors  while doing it. RB must
  'not do anything else in the meantime since
  'the system would not respond for 20 secs.
  'The RB program must do a 20 seconds delay
  'to allow for it
  'If the Mode is 0 then the command will
  'return immediately. RB can then do its own
  'motor turning if it wants.
  'The calibration will be done in the background.  
  if Not Compass_On  or Compass_Calibration
     return     
  Compass.StartCalibration
  if Mode == 0
     Compass_Timer := cnt               'start timer for end of calibration
     Compass_Calibration~~              'indicate that calibration is on
     'the endcalibration is executed when the time finishes
     'in the Process_0 method
  else
     Compass_Timer := cnt
     repeat until cnt-Compass_Timer > ClkFreq*20   
        byte[Command][0] := 12
        Byte[Command][1] := 10
        byte[Sems_Flags][1] |= MotorsFlagMask   'Flag the Motors Cog 
     Compass.EndCalibration

Pri Ping(Pin) : MS| x,time[2]
  outA[Pin]~               'set Pin low
  dirA[Pin]~~              'make Pin Output
  outA[Pin]~~              'Set Pin high
  outA[Pin]~               'Set Pin low ( results in a > 2 s pulse)
  dirA[Pin]~               'make Pin Input
  x:=cnt                   'start timing for timeout 
  repeat until inA[Pin]    'wait for Pin to go high
    if cnt-x > clkfreq/100 'with a 10ms time out
      return 0             'if timed out then return 0
  time[0] := cnt           'store Current Counter Value
  waitpeq(0, |< Pin, 0)    'wait For Pin To Go LOW 
  time[1] := cnt           'store New Counter Value
  MS :=||(time[0] - time[1])/(clkfreq / 1_000_000) ' Return Time in s