//Simplistic_INS.Bas
//works with Protocol_Main.Spin

Port = 16 //change this as per your system
#include "..\Utilities&IncludeFiles\Instruments.Bas"
//---------------------------------------
Main:
  rcommport Port,br115200
  rLocate 600,300
  gosub Calibrate         //calibrate
  Call CollectTheData()   //Collect the data
  Call Calculate()        //do calculations 
  Call SaveToFile()       //save all the results to a disk file
  Call Plot()             //graph the data
End
//=================================
sub SendCommand(C,P,&s)
  m = ""
  s = rCommand(C,P) \ x = Length(s)
  if x < 5 then m= "Comms Error"
  xystring 500,20,spaces(30)  
return (x == 5)
//=================================
Calibrate:
  print "Calibrating....please wait"
  Sgx = 0 \ Sgy=0 \ N = 100
  for i=0 to N-1
     call Read_H48C(0,,,,gX,gY)  //read N readings
     Sgx += gX \ Sgy += gY
  next
  data values;Sgx/N,Sgy/N  //initialize array with average
  clearscr
Return
//==================================
sub CollectTheData()
  clearscr
  print "When the tone stops start moving the Board"
  print "........Press Finish when done......"  
  rCommand(73,70) \ delay 3000        //sound tone and wait 4 secs
  rCommand(73,0)                      //stop the tone
  addbutton "&Finish",300,300 \ FocusButton "&Finish"
  data values;timer()                 //starting time
  for i=0 to 5000                     //limit to 5000 readings (about 4 minutes)
    call Read_H48C(0,,,,gX,gY)        //read the accelerations
    data values;gX,gY,timer()         //insert in an array
    if lastbutton() != "" then break  //exit if button pushed
  next   
  RemoveButton "&Finish"
Return
//==================================
sub Calculate()
  ClearScr \   Print "Calculating.... Please Wait"
  data T;0  \  data Ax;0  \  data Ay;0     //init arrays
  data Vx;0 \  data Vy;0  \  data Sx;0  \ data Sy;0
  for i=3 to maxdim(values)-1 step 3
    I = i/3 \ J=I-1
    data T;(values[i+2]-values[2])/1000    //timing array
    dt = (values[i+2]-values[i-1])/1000 \ dt2 = dt*dt/2
    data Ax;(values[i]-values[0])*9.81     //Acceleration
    data Ay;(values[i+1]-values[1])*9.81
    data Vx; Ax[I]*dt+Vx[J]                //V[i]=A*dt+V[i-1]
    data Vy; Ay[I]*dt+Vy[J]
    data Sx; Ax[I]*dt2+Vx[J]*dt+Sx[J]      //S[i]=(A*dt^2)/2+V[i-1]*dt+S[i-1]
    data Sy; Ay[I]*dt2+Vy[J]*dt+Sy[J]  
  next
return
//==================================
sub Calculate2()
  ClearScr \   Print "Calculating.... Please Wait"
  data T;0  \  data Ax;0  \  data Ay;0                //init arrays
  data Vx;0 \  data Vy;0  \  data Sx;0  \ data Sy;0
  n = Maxdim(values)/3 
  dim AccX[2,n], AccY[2,n]
  mconstant AccX,0 \ mconstant AccY,0
  for i=3 to maxdim(values)-1 step 3
    I = i/3 \ J=I-1 \ t = (values[i+2]-values[2])/1000
    AccX[0,I] = t \ AccY[0,I] = t \ data T;t
    AccX[1,I] = (values[i]-values[0])*9.81
    AccY[1,I] = (values[i+1]-values[1])*9.81
  next
  m = 5  \ data CoefficientsX;m   //change m to 2 and 3 and 4 to see the effects
  data CoefficientsY;m
  mPolyFit AccX,CoefficientsX
  mPolyFit AccY,CoefficientsY
  for I=1 to n-1
    ax = 0 \ ay = 0 \ J = I-1
    for j=0 to m
      ax = ax+CoefficientsX[j]*T[I]^j
      ay = ay+CoefficientsY[j]*T[I]^j
    next
    dt = T[I]-T[J] \ dt2 = dt*dt/2
    data Ax;ax \ data Ay;ay
    data Vx; Ax[I]*dt+Vx[J]                //V[i]=A*dt+V[i-1]
    data Vy; Ay[I]*dt+Vy[J]
    data Sx; Ax[I]*dt2+Vx[J]*dt+Sx[J]      //S[i]=(A*dt^2)/2+V[i-1]*dt+S[i-1]
    data Sy; Ay[I]*dt2+Vy[J]*dt+Sy[J]  
  next
Return
//==================================
sub SaveToFile()
  s = "Time,Ax,Vx,Sx,Ay,Vy,Sy"+crlf()
  for i=0 to maxdim(T)-1
     s += ""+T[i]+","+Ax[i]+","+Vx[i]+","+Sx[i]
     s += ","+Ay[i]+","+Vy[i]+","+Sy[i]+crlf()
  next
  mFromString tA,s
  mTextFW tA,"Simplistic_INS.CSV"
return
//==================================
sub Plot()
  //plot the graphs
  Clearscr  \fnt = "Times New Roman"
  data GPx_Specs;10,100,380,400
  mGraphPaper GPx_Specs   //graph paper
  data GPy_Specs;405,100,380,400
  mGraphPaper GPy_Specs
  //label the graphs
  for i=0 to 1
     xyText 150+400*i,65,"A"+char(ascii("x")+i)+", ",fnt,20,fs_Bold,red
     xyText ,,"V"+char(ascii("x")+i)+", ",fnt,20,fs_Bold,blue
     xyText ,,"S"+char(ascii("x")+i),fnt,20,fs_Bold,black
     xyText 400*i,80,"5,",fnt,10,fs_Bold,red  
     xyText ,,"1,",fnt,10,fs_Bold,blue
     xyText ,,"2",fnt,10,fs_Bold,black
     xyText 400*i,505,"-5,",fnt,10,fs_Bold,red  
     xyText ,,"-1,",fnt,10,fs_Bold,blue
     xyText ,,"-2",fnt,10,fs_Bold,black
     xyText 140+i*400,515,"Time (Secs)",fnt,20,fs_Bold
  next   
  mT = max(T)
  for i=0 to 1
    xyText 10+i*400,525,"0",fnt,10,fs_Bold     
    xyText 350+i*400,525,Format(mT,"###0.0##"),fnt,10,fs_Bold     
  next
  //plot Ax,Vx,Sx
  m=maxv(maxv(abs(Max(Ax)),abs(Min(Ax))),5)
  Data GxA_Specs;10,100,370,400,red,,,,-m,m    //graph specs..force limits
  mPlotXY GxA_Specs,T,Ax                       //plot acceleration against time
  m=maxv(maxv(abs(Max(Vx)),abs(Min(Vx))),1)
  Data GxV_Specs;10,100,370,400,,,,,-m,m       //graph specs..force limits
  mPlotXY GxV_Specs,T,Vx                       //plot the velocity against time
  m=maxv(maxv(abs(Max(Sx)),abs(Min(Sx))),2)
  Data GxS_Specs;10,100,370,400,Black,,,,-m,m  //graph specs..force limits
  mPlotXY GxS_Specs,T,Sx                       //plot displacement against time
  //plot Ay,Vy,Sy
  m=maxv(maxv(abs(Max(Ay)),abs(Min(Ay))),5)
  Data GyA_Specs;405,100,370,400,red,,,,-m,m   //graph specs..force limits     
  mPlotXY GyA_Specs,T,Ay                       //plot acceleration against time
  m=maxv(maxv(abs(Max(Vy)),abs(Min(Vy))),1)
  Data GyV_Specs;405,100,370,400,,,,,-m,m      //graph specs..force limits
  mPlotXY GyV_Specs,T,Vy                       //plot the velocity against time
  m=maxv(maxv(abs(Max(Sy)),abs(Min(Sy))),2)
  Data GyS_Specs;405,100,370,400,Black,,,,-m,m //graph specs..force limits
  mPlotXY GyS_Specs,T,Sy                       //plot displacement against time
Return 
